package com.inkasso.factoring.bocom.register.service.impl;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.inkasso.adapter.bean.AbstractUser;
import com.inkasso.adapter.exception.BusinessException;
import com.inkasso.adapter.util.BeanUtilsProxy;
import com.inkasso.adapter.util.CodeUtil;
import com.inkasso.adapter.util.CollectionUtil;
import com.inkasso.adapter.util.StringUtil;
import com.inkasso.adapter.web.LoginUserHolder;
import com.inkasso.auth.response.ResponseJson;
import com.inkasso.factoring.bocom.register.dao.BocomBasicInfoMapper;
import com.inkasso.factoring.bocom.register.entity.*;
import com.inkasso.factoring.bocom.register.entity.vo.BocomSearchVo;
import com.inkasso.factoring.bocom.register.entity.vo.EnterpriseDetailVo;
import com.inkasso.factoring.bocom.register.enums.BocomCustomerStageEnums;
import com.inkasso.factoring.bocom.register.service.*;
import com.inkasso.factoring.dict.entity.Dictionary;
import com.inkasso.factoring.dict.service.DictionaryService;
import com.inkasso.factoring.file.entity.FileStorageDO;
import com.inkasso.factoring.file.enums.FileTypeEnum;
import com.inkasso.factoring.file.service.FileService;
import com.inkasso.factoring.system.user.entity.UserDO;
import com.inkasso.factoring.system.user.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 交通银行客户信息 服务实现类
 * </p>
 *
 * @author liutao
 * @since 2024-07-29
 */
@Service
public class BocomBasicInfoServiceImpl extends ServiceImpl<BocomBasicInfoMapper, BocomBasicInfo> implements IBocomBasicInfoService {

    @Autowired
    private IBocomBasicInfoHistoryService bocomBasicInfoHistoryService;

    @Autowired
    private IBocomLegalPersonInfoService bocomLegalPersonInfoService;

    @Autowired
    private IBocomLegalPersonInfoHistoryService bocomLegalPersonInfoHistoryService;

    @Autowired
    private IBocomShareholderActualControllerService bocomShareholderActualControllerService;

    @Autowired
    private IBocomShareholderActualControllerHistoryService bocomShareholderActualControllerHistoryService;

    @Autowired
    private IBocomBeneficiaryInfoService bocomBeneficiaryInfoService;

    @Autowired
    private IBocomBeneficiaryInfoHistoryService bocomBeneficiaryInfoHistoryService;

    @Autowired
    private IBocomAuthorizedAgentService bocomAuthorizedAgentService;

    @Autowired
    private IBocomAuthorizedAgentHistoryService bocomAuthorizedAgentHistoryService;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private FileService fileService;

    @Autowired
    private UserService userService;

    @Autowired
    private IBocomApprovalService bocomApprovalService;

    @Value("${trade.bocomRegisterUrl}")
    private String url;

    @Value("${sop.getEnterpriseDetail}")
    private String getEnterpriseDetail;


    @Autowired
    private DictionaryService dictionaryService;

    @Value("${trade.bocomRegisterResetUrl}")
    private String bocomRegisterResetUrl;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(BocomBasicInfo bocomBasicInfo) {
        if (bocomBasicInfo.getId() == null) {
            Long count = this.lambdaQuery().notIn(BocomBasicInfo::getCustomerStatus, BocomCustomerStageEnums.FAIL.getCode()).eq(BocomBasicInfo::getSocialCreditCode, bocomBasicInfo.getSocialCreditCode()).count();
            if (count > 0) {
                throw new BusinessException(CodeUtil.FAIL, "该用户有在途的开户数据！");
            }
        }
        bocomBasicInfo.setCustomerSource("贸理通C");
        //修改主表信息
        this.saveOrUpdate(bocomBasicInfo);
        //删除之前的文件
        fileService.lambdaUpdate().eq(FileStorageDO::getBusinessType, FileTypeEnum.BOCOM_REGISTER_BUSINESS_LICENSE.getCode()).eq(FileStorageDO::getBusinessId, bocomBasicInfo.getId()).set(FileStorageDO::getBusinessId, null).update();
        Long fileId = bocomBasicInfo.getFileId();
        Long front1 = bocomBasicInfo.getFront();
        Long back1 = bocomBasicInfo.getBack();
        if (fileId != null) {
            FileStorageDO fileStorageDO = fileService.getById(fileId);
            fileStorageDO.setId(null);
            fileStorageDO.setBusinessId(bocomBasicInfo.getId());
            fileService.save(fileStorageDO);
        }
        if (front1 != null) {
            FileStorageDO fileStorageDO = fileService.getById(front1);
            fileStorageDO.setId(null);
            fileStorageDO.setBusinessId(bocomBasicInfo.getId());
            fileService.save(fileStorageDO);
        }
        if (back1 != null) {
            FileStorageDO fileStorageDO = fileService.getById(back1);
            fileStorageDO.setId(null);
            fileStorageDO.setBusinessId(bocomBasicInfo.getId());
            fileService.save(fileStorageDO);
        }
        //删除历史数据
        bocomLegalPersonInfoService.lambdaUpdate().eq(BocomLegalPersonInfo::getBocomBasicInfoId, bocomBasicInfo.getId()).remove();
        bocomShareholderActualControllerService.lambdaUpdate().eq(BocomShareholderActualController::getBocomBasicInfoId, bocomBasicInfo.getId()).remove();
        bocomBeneficiaryInfoService.lambdaUpdate().eq(BocomBeneficiaryInfo::getBocomBasicInfoId, bocomBasicInfo.getId()).remove();
        bocomAuthorizedAgentService.lambdaUpdate().eq(BocomAuthorizedAgent::getBocomBasicInfoId, bocomBasicInfo.getId()).remove();
        //保存新数据
        BocomLegalPersonInfo bocomLegalPersonInfo = bocomBasicInfo.getBocomLegalPersonInfo();
        if (bocomLegalPersonInfo != null) {
            bocomLegalPersonInfo.setBocomBasicInfoId(bocomBasicInfo.getId());
            bocomLegalPersonInfoService.save(bocomLegalPersonInfo);
            Long front = bocomLegalPersonInfo.getFront();
            if (front != null) {
                FileStorageDO fileStorageDO = fileService.getById(front);
                fileStorageDO.setId(null);
                fileStorageDO.setBusinessId(bocomLegalPersonInfo.getId());
                fileService.save(fileStorageDO);
            }
            Long back = bocomLegalPersonInfo.getBack();
            if (back != null) {
                FileStorageDO fileStorageDOBack = fileService.getById(back);
                fileStorageDOBack.setId(null);
                fileStorageDOBack.setBusinessId(bocomLegalPersonInfo.getId());
                fileService.save(fileStorageDOBack);
            }
        }
        List<BocomShareholderActualController> bocomShareholderActualControllers = bocomBasicInfo.getBocomShareholderActualControllers();
        if (CollectionUtil.isNotEmpty(bocomShareholderActualControllers)) {
            for (BocomShareholderActualController bocomShareholderActualController : bocomShareholderActualControllers) {
                bocomShareholderActualController.setBocomBasicInfoId(bocomBasicInfo.getId());
                bocomShareholderActualControllerService.save(bocomShareholderActualController);
                Long front = bocomShareholderActualController.getFront();
                if (front != null) {
                    FileStorageDO fileStorageDO = fileService.getById(front);
                    fileStorageDO.setId(null);
                    fileStorageDO.setBusinessId(bocomShareholderActualController.getId());
                    fileStorageDO.setBusinessType(FileTypeEnum.BOCOM_SHAREHOLDER_IDCARD_FRONT.getCode());
                    fileService.save(fileStorageDO);
                }
                Long back = bocomShareholderActualController.getBack();
                if (back != null) {
                    FileStorageDO fileStorageDOBack = fileService.getById(back);
                    fileStorageDOBack.setId(null);
                    fileStorageDOBack.setBusinessId(bocomShareholderActualController.getId());
                    fileStorageDOBack.setBusinessType(FileTypeEnum.BOCOM_SHAREHOLDER_IDCARD_BACK.getCode());
                    fileService.save(fileStorageDOBack);
                }
            }
        }
        List<BocomBeneficiaryInfo> bocomBeneficiaryInfos = bocomBasicInfo.getBocomBeneficiaryInfos();
        if (CollectionUtil.isNotEmpty(bocomBeneficiaryInfos)) {
            for (BocomBeneficiaryInfo bocomBeneficiaryInfo : bocomBeneficiaryInfos) {
                bocomBeneficiaryInfo.setBocomBasicInfoId(bocomBasicInfo.getId());
                bocomBeneficiaryInfoService.save(bocomBeneficiaryInfo);
                Long front = bocomBeneficiaryInfo.getFront();
                if (front != null) {
                    FileStorageDO fileStorageDO = fileService.getById(front);
                    fileStorageDO.setId(null);
                    fileStorageDO.setBusinessId(bocomBeneficiaryInfo.getId());
                    fileStorageDO.setBusinessType(FileTypeEnum.BOCOM_BENEFICIARY_IDCARD_FRONT.getCode());
                    fileService.save(fileStorageDO);
                }
                Long back = bocomBeneficiaryInfo.getBack();
                if (back != null) {
                    FileStorageDO fileStorageDOBack = fileService.getById(back);
                    fileStorageDOBack.setId(null);
                    fileStorageDOBack.setBusinessId(bocomBeneficiaryInfo.getId());
                    fileStorageDOBack.setBusinessType(FileTypeEnum.BOCOM_BENEFICIARY_IDCARD_BACK.getCode());
                    fileService.save(fileStorageDOBack);
                }
            }
        }
        BocomAuthorizedAgent bocomAuthorizedAgent = bocomBasicInfo.getBocomAuthorizedAgent();
        if (bocomAuthorizedAgent != null) {
            bocomAuthorizedAgent.setBocomBasicInfoId(bocomBasicInfo.getId());
            bocomAuthorizedAgentService.save(bocomAuthorizedAgent);
            Long front = bocomAuthorizedAgent.getFront();
            if (front != null) {
                FileStorageDO fileStorageDO = fileService.getById(front);
                fileStorageDO.setId(null);
                fileStorageDO.setBusinessId(bocomAuthorizedAgent.getId());
                fileStorageDO.setBusinessType(FileTypeEnum.BOCOM_AUTHORIZED_AGENT_IDCARD_FRONT.getCode());
                fileService.save(fileStorageDO);
            }
            Long back = bocomAuthorizedAgent.getBack();
            if (back != null) {
                FileStorageDO fileStorageDOBack = fileService.getById(back);
                fileStorageDOBack.setId(null);
                fileStorageDOBack.setBusinessId(bocomAuthorizedAgent.getId());
                fileStorageDOBack.setBusinessType(FileTypeEnum.BOCOM_AUTHORIZED_AGENT_IDCARD_BACK.getCode());
                fileService.save(fileStorageDOBack);
            }
        }
        if (bocomBasicInfo.getVisible()) {
            bocomBasicInfo.setStage(BocomCustomerStageEnums.REVIEW.getCode());
            this.updateById(bocomBasicInfo);
            //保存历史
            Long historyId = addHistory(bocomBasicInfo);
            bocomBasicInfo.setHistoryId(historyId);
            //B端流程
            restTemplate.postForObject(url, bocomBasicInfo, ResponseJson.class);
        } else {
            bocomBasicInfo.setStage(BocomCustomerStageEnums.STAGING.getCode());
            this.updateById(bocomBasicInfo);
        }
    }

    @Override
    public BocomBasicInfo getDetail(Long id) {
        //基础信息
        BocomBasicInfo bocomBasicInfo = getById(id);
        FileStorageDO file = fileService.findOneByBusinessId(id, FileTypeEnum.BOCOM_REGISTER_BUSINESS_LICENSE.getCode());
        FileStorageDO customerFront = fileService.findOneByBusinessId(id, FileTypeEnum.BOCOM_IDCARD_FRONT.getCode());
        FileStorageDO customerBack = fileService.findOneByBusinessId(id, FileTypeEnum.BOCOM_IDCARD_BACK.getCode());
        bocomBasicInfo.setFile(file);
        bocomBasicInfo.setFrontFile(customerFront);
        bocomBasicInfo.setBackFile(customerBack);
        BocomLegalPersonInfo bocomLegalPersonInfo = bocomLegalPersonInfoService.lambdaQuery().eq(BocomLegalPersonInfo::getBocomBasicInfoId, bocomBasicInfo.getId()).one();
        if (bocomLegalPersonInfo != null) {
            FileStorageDO front = fileService.findOneByBusinessId(bocomLegalPersonInfo.getId(), FileTypeEnum.BOCOM_LEGAL_REPRESENTATIVE_IDCARD_FRONT.getCode());
            FileStorageDO back = fileService.findOneByBusinessId(bocomLegalPersonInfo.getId(), FileTypeEnum.BOCOM_LEGAL_REPRESENTATIVE_IDCARD_BACK.getCode());
            bocomLegalPersonInfo.setFrontFile(front);
            bocomLegalPersonInfo.setBackFile(back);
            bocomBasicInfo.setBocomLegalPersonInfo(bocomLegalPersonInfo);
        }
        List<BocomShareholderActualController> bocomShareholderActualControllers = bocomShareholderActualControllerService.lambdaQuery().eq(BocomShareholderActualController::getBocomBasicInfoId, bocomBasicInfo.getId()).list();
        if (CollectionUtil.isNotEmpty(bocomShareholderActualControllers)) {
            for (BocomShareholderActualController bocomShareholderActualController : bocomShareholderActualControllers) {
                FileStorageDO front = fileService.findOneByBusinessId(bocomShareholderActualController.getId(), FileTypeEnum.BOCOM_SHAREHOLDER_IDCARD_FRONT.getCode());
                FileStorageDO back = fileService.findOneByBusinessId(bocomShareholderActualController.getId(), FileTypeEnum.BOCOM_SHAREHOLDER_IDCARD_BACK.getCode());
                bocomShareholderActualController.setFrontFile(front);
                bocomShareholderActualController.setBackFile(back);
            }
            bocomBasicInfo.setBocomShareholderActualControllers(bocomShareholderActualControllers);
        }
        List<BocomBeneficiaryInfo> bocomBeneficiaryInfos = bocomBeneficiaryInfoService.lambdaQuery().eq(BocomBeneficiaryInfo::getBocomBasicInfoId, bocomBasicInfo.getId()).list();
        if (CollectionUtil.isNotEmpty(bocomBeneficiaryInfos)) {
            for (BocomBeneficiaryInfo bocomBeneficiaryInfo : bocomBeneficiaryInfos) {
                FileStorageDO front = fileService.findOneByBusinessId(bocomBeneficiaryInfo.getId(), FileTypeEnum.BOCOM_BENEFICIARY_IDCARD_FRONT.getCode());
                FileStorageDO back = fileService.findOneByBusinessId(bocomBeneficiaryInfo.getId(), FileTypeEnum.BOCOM_BENEFICIARY_IDCARD_BACK.getCode());
                bocomBeneficiaryInfo.setFrontFile(front);
                bocomBeneficiaryInfo.setBackFile(back);
            }
            bocomBasicInfo.setBocomBeneficiaryInfos(bocomBeneficiaryInfos);
        }
        BocomAuthorizedAgent bocomAuthorizedAgent = bocomAuthorizedAgentService.lambdaQuery().eq(BocomAuthorizedAgent::getBocomBasicInfoId, bocomBasicInfo.getId()).one();
        if (bocomAuthorizedAgent != null) {
            FileStorageDO front = fileService.findOneByBusinessId(bocomAuthorizedAgent.getId(), FileTypeEnum.BOCOM_AUTHORIZED_AGENT_IDCARD_FRONT.getCode());
            FileStorageDO back = fileService.findOneByBusinessId(bocomAuthorizedAgent.getId(), FileTypeEnum.BOCOM_AUTHORIZED_AGENT_IDCARD_BACK.getCode());
            bocomAuthorizedAgent.setFrontFile(front);
            bocomAuthorizedAgent.setBackFile(back);
            bocomBasicInfo.setBocomAuthorizedAgent(bocomAuthorizedAgent);
        }
        return bocomBasicInfo;
    }

    @Override
    public List<BocomBasicInfo> findBySearch(BocomSearchVo bocomSearchVo) {
        AbstractUser abstractUser = LoginUserHolder.get();
        UserDO userDO = userService.getById(abstractUser.getId());
        List<BocomBasicInfo> list = this.lambdaQuery().in(BocomBasicInfo::getUserCreate, userDO.getLoginName(), userDO.getUserName()).list();
        for (BocomBasicInfo bocomBasicInfo : list) {
            Dictionary stage = dictionaryService.getByCode(bocomBasicInfo.getStage());
            Dictionary status = dictionaryService.getByCode(bocomBasicInfo.getCustomerStatus());
            if (stage != null) {
                bocomBasicInfo.setStage(stage.getName());
            }
            if (status != null) {
                bocomBasicInfo.setCustomerStatus(status.getName());
            }
            List<BocomApproval> bocomApprovals = bocomApprovalService.lambdaQuery().eq(BocomApproval::getBocomBasicInfoId, bocomBasicInfo.getId()).orderByDesc(BocomApproval::getGmtCreate).list();
            if (CollectionUtil.isNotEmpty(bocomApprovals)) {
                BocomApproval bocomApproval = bocomApprovals.get(0);
                if (bocomApproval != null && bocomApproval.getApprovalConclusion().equals("back")) {
                    bocomBasicInfo.setReason(bocomApproval.getApprovalComment());
                }
            }

            //填充法人授权人
            if (bocomBasicInfo.getCustomerType() == 0) {
                BocomLegalPersonInfo legalPersonInfo = bocomLegalPersonInfoService.lambdaQuery().eq(BocomLegalPersonInfo::getBocomBasicInfoId, bocomBasicInfo.getId())
                        .one();
                if (legalPersonInfo != null) {
                    bocomBasicInfo.setAuthenticator(legalPersonInfo.getName());
                }
            } else if (bocomBasicInfo.getCustomerType() == 1) {
                BocomShareholderActualController actualController = bocomShareholderActualControllerService.lambdaQuery().eq(BocomShareholderActualController::getBocomBasicInfoId, bocomBasicInfo.getId())
                        .eq(BocomShareholderActualController::getPersonType, 2)
                        .one();
                if (actualController != null){
                    bocomBasicInfo.setAuthenticator(actualController.getName());
                }
            }
        }
        return list;
    }

    @Override
    public EnterpriseDetailVo getBasicDetailsByName(String keyword) {
        //2003 客户身份识别
        ResponseJson responseJson = restTemplate.getForObject(getEnterpriseDetail + "?keyword=" + keyword, ResponseJson.class);
        JSONObject data = new JSONObject(responseJson.getData());
        String date = data.getStr("date");
        if (StringUtil.isEmpty(date)) {
            date = "99991231";
        }
        String scope = data.getStr("scope");
        EnterpriseDetailVo enterpriseDetailVo = new EnterpriseDetailVo();
        enterpriseDetailVo.setDate(date);
        enterpriseDetailVo.setScope(scope);
        //股东信息(工商登记)
        List<JSONObject> partnerList = new ArrayList<>();
        JSONArray partnerArray = data.getJSONArray("partnerList");
        if (CollectionUtil.isNotEmpty(partnerArray)) {
            for (Object o : partnerArray) {
                String name = (String) o;
                if (name.length() <= 4) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.set("name", name);
                    jsonObject.set("type", 0);
                    partnerList.add(jsonObject);
                }
            }
        }
        if (CollectionUtil.isEmpty(partnerList)) {
            //实际控制人
            JSONArray actualControlArray = data.getJSONArray("actualControlList");
            if (CollectionUtil.isNotEmpty(actualControlArray)) {
                for (Object o : actualControlArray) {
                    String name = (String) o;
                    if (name.length() <= 4) {
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.set("name", name);
                        jsonObject.set("type", 1);
                        partnerList.add(jsonObject);
                    }
                }
            }
        }
        enterpriseDetailVo.setPartnerList(partnerList);
        //受益所有人
        List<String> beneficialOwnerList = new ArrayList<>();
        JSONArray beneficialOwnerArray = data.getJSONArray("beneficialOwnerList");
        if (CollectionUtil.isNotEmpty(beneficialOwnerArray)) {
            for (Object o : beneficialOwnerArray) {
                String name = (String) o;
                if (name.length() <= 4) {
                    beneficialOwnerList.add(name);
                }
            }
        }
        enterpriseDetailVo.setBeneficialOwnerList(beneficialOwnerList);
        return enterpriseDetailVo;
    }

    @Override
    public List<String> getReason(Long id) {
        BocomBasicInfo bocomBasicInfo = getById(id);
        String vrfyFailStsWord = bocomBasicInfo.getVrfyFailStsWord();
        List<String> reason = new ArrayList<>();
        if (StringUtil.isNotEmpty(vrfyFailStsWord)) {
            char charToFind = '1';
            int index = vrfyFailStsWord.indexOf(charToFind);
            if (index >= 0) {
                String reasons = findReasons(index);
                reason.add(reasons);
            }
            while (index >= 0) {
                index = vrfyFailStsWord.indexOf(charToFind, index + 1);
                if (index >= 0) {
                    String reasons = findReasons(index);
                    reason.add(reasons);
                }
            }
        }
        return reason;
    }

    @Override
    public BocomBasicInfo resetUrlAuthorized(Long id) {

        try {
            com.inkasso.factoring.common.entity.ResponseJson responseJson = restTemplate.getForObject(bocomRegisterResetUrl + "?id=" + id, com.inkasso.factoring.common.entity.ResponseJson.class);
            if (!responseJson.getCode().equals(200)) {
                throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
            }
        } catch (Exception e) {
            throw new BusinessException(CodeUtil.FAIL, e.getMessage());
        }
        BocomBasicInfo basicInfo = getById(id);
        //填充法人授权人
        if (basicInfo.getCustomerType() == 0) {
            BocomLegalPersonInfo legalPersonInfo = bocomLegalPersonInfoService.lambdaQuery().eq(BocomLegalPersonInfo::getBocomBasicInfoId, id)
                    .one();
            if (legalPersonInfo != null) {
                basicInfo.setAuthenticator(legalPersonInfo.getName());
            }
        } else if (basicInfo.getCustomerType() == 1) {
            BocomShareholderActualController actualController = bocomShareholderActualControllerService.lambdaQuery().eq(BocomShareholderActualController::getBocomBasicInfoId, id)
                    .eq(BocomShareholderActualController::getPersonType, 2)
                    .one();
            if (actualController != null){
                basicInfo.setAuthenticator(actualController.getName());
            }
        }
        return basicInfo;
    }

    private String findReasons(int index) {
        int number = index + 1;
        String result = "";
        if (number == 1) {
            result = "营业执照影像文件docid错误";
        } else if (number == 2) {
            result = "营业执照影像文件OCR识别未通过";
        } else if (number == 3) {
            result = "个人证件影像文件docid错误";
        } else if (number == 4) {
            result = "个人证件影像文件OCR识别未通过";
        } else if (number == 5) {
            result = "法定代表人/负责人证件影像文件docid错误";
        } else if (number == 6) {
            result = "法定代表人/负责人证件影像文件OCR识别未通过";
        } else if (number == 7) {
            result = "受益人证件影像文件docid错误";
        } else if (number == 8) {
            result = "受益人证件影像文件OCR识别未通过";
        } else if (number == 9) {
            result = "控制人证件影像文件docid错误";
        } else if (number == 10) {
            result = "控制人证件影像文件OCR识别未通过";
        } else if (number == 11) {
            result = "股东证件影像文件docid错误";
        } else if (number == 12) {
            result = "股东证件影像文件OCR识别未通过";
        } else if (number == 13) {
            result = "授权代理人证件影像文件docid错误";
        } else if (number == 14) {
            result = "授权代理人证件影像文件OCR识别未通过";
        } else if (number == 15) {
            result = "公司客户工商联网校验未通过";
        } else if (number == 16) {
            result = "公司客户红盾校验失败";
        } else if (number == 17) {
            result = "客户反洗钱特殊名单未通过";
        } else if (number == 18) {
            result = "客户反洗钱风险等级较高";
        } else if (number == 19) {
            result = "客户电信诈骗黑名单校验未通过";
        } else if (number == 20) {
            result = "法定代表人/负责人公安联网核查未通过";
        } else if (number == 21) {
            result = "法定代表人/负责人反洗钱特殊名单未通过";
        } else if (number == 22) {
            result = "法定代表人/负责人反洗钱风险等级较高";
        } else if (number == 23) {
            result = "法定代表人/负责人手机实名制认证失败";
        } else if (number == 24) {
            result = "个人客户公安联网核查失败";
        } else if (number == 25) {
            result = "个人手机实名制认证失败";
        } else if (number == 26) {
            result = "授权代理人风险等级较高";
        } else if (number == 27) {
            result = "授权代理人公安联网核查失败";
        } else if (number == 28) {
            result = "授权代理人手机实名制认证失败";
        } else if (number == 29) {
            result = "授权代理人反洗钱特殊名单未通过";
        } else if (number == 30) {
            result = "受益人公安联网核查失败";
        } else if (number == 31) {
            result = "受益人反洗钱特殊名单未通过";
        } else if (number == 32) {
            result = "受益人反洗钱风险等级较高";
        } else if (number == 33) {
            result = "受益人电信诈骗核验失败";
        } else if (number == 34) {
            result = "国际负面清单";
        } else if (number == 35) {
            result = "控制人公安联网核查失败";
        } else if (number == 36) {
            result = "控制人反洗钱特殊名单未通过";
        } else if (number == 37) {
            result = "控制人反洗钱风险等级较高";
        } else if (number == 38) {
            result = "控制人电信诈骗核验失败";
        } else if (number == 39) {
            result = "股东公安联网核查失败（股东为个人时）";
        } else if (number == 40) {
            result = "股东反洗钱特殊名单未通过";
        } else if (number == 41) {
            result = "股东反洗钱风险等级较高";
        } else if (number == 42) {
            result = "股东电信诈骗核验失败";
        } else if (number == 43) {
            result = "股东工商校验未通过（股东为公司时）";
        } else if (number == 44) {
            result = "企业会员法人业务授权docid错误";
        } else if (number == 45) {
            result = "授权代理人个保授权docid错误";
        } else if (number == 46) {
            result = "法人/负责人个人信息使用授权docid错误";
        } else if (number == 47) {
            result = "个人业务授权docid错误";
        } else if (number == 48) {
            result = "个人个保授权docid错误）";
        } else if (number == 49) {
            result = "法人电信诈骗黑名单校验未通过";
        } else if (number == 50) {
            result = "负责人电信诈骗黑名单校验未通";
        } else if (number == 51) {
            result = "授权代理人电信诈骗黑名单校验未通过";
        } else if (number == 52) {
            result = "授权代理人业务授权docid错误";
        }
        return result;
    }

    private Long addHistory(BocomBasicInfo bocomBasicInfo) {
        //主表信息
        BocomBasicInfoHistory bocomBasicInfoHistory = BeanUtilsProxy.copyProperties(bocomBasicInfo, BocomBasicInfoHistory.class);
        bocomBasicInfoHistory.setId(null);
        bocomBasicInfoHistoryService.save(bocomBasicInfoHistory);
        Long fileId = bocomBasicInfo.getFileId();
        Long front1 = bocomBasicInfo.getFront();
        Long back1 = bocomBasicInfo.getBack();
        if (front1 != null) {
            FileStorageDO existsFile = fileService.getById(front1);
            existsFile.setId(null);
            existsFile.setBusinessType(FileTypeEnum.BOCOM_IDCARD_FRONT_HISTORY.getCode());
            existsFile.setBusinessId(bocomBasicInfoHistory.getId());
            fileService.save(existsFile);
        }
        if (back1 != null) {
            FileStorageDO existsFile = fileService.getById(back1);
            existsFile.setId(null);
            existsFile.setBusinessType(FileTypeEnum.BOCOM_IDCARD_BACK_HISTORY.getCode());
            existsFile.setBusinessId(bocomBasicInfoHistory.getId());
            fileService.save(existsFile);
        }
        if (fileId != null) {
            FileStorageDO existsFile = fileService.getById(fileId);
            existsFile.setId(null);
            existsFile.setBusinessType(FileTypeEnum.BOCOM_REGISTER_BUSINESS_LICENSE_HISTORY.getCode());
            existsFile.setBusinessId(bocomBasicInfoHistory.getId());
            fileService.save(existsFile);
        }
        //法定代表人信息
        BocomLegalPersonInfo bocomLegalPersonInfo = bocomBasicInfo.getBocomLegalPersonInfo();
        if (bocomLegalPersonInfo != null) {
            BocomLegalPersonInfoHistory bocomLegalPersonInfoHistory = BeanUtilsProxy.copyProperties(bocomLegalPersonInfo, BocomLegalPersonInfoHistory.class);
            bocomLegalPersonInfoHistory.setId(null);
            bocomLegalPersonInfoHistory.setBocomBasicInfoId(bocomBasicInfoHistory.getId());
            bocomLegalPersonInfoHistoryService.save(bocomLegalPersonInfoHistory);
            Long front = bocomLegalPersonInfo.getFront();
            if (front != null) {
                FileStorageDO frontFile = fileService.getById(front);
                frontFile.setId(null);
                frontFile.setBusinessType(FileTypeEnum.BOCOM_LEGAL_REPRESENTATIVE_IDCARD_FRONT_HISTORY.getCode());
                frontFile.setBusinessId(bocomLegalPersonInfoHistory.getId());
                fileService.save(frontFile);
            }
            Long back = bocomLegalPersonInfo.getBack();
            if (back != null) {
                FileStorageDO backFile = fileService.getById(back);
                backFile.setId(null);
                backFile.setBusinessType(FileTypeEnum.BOCOM_LEGAL_REPRESENTATIVE_IDCARD_BACK_HISTORY.getCode());
                backFile.setBusinessId(bocomLegalPersonInfoHistory.getId());
                fileService.save(backFile);
            }
        }
        List<BocomShareholderActualController> bocomShareholderActualControllers = bocomBasicInfo.getBocomShareholderActualControllers();
        if (CollectionUtil.isNotEmpty(bocomShareholderActualControllers)) {
            for (BocomShareholderActualController bocomShareholderActualController : bocomShareholderActualControllers) {
                BocomShareholderActualControllerHistory bocomShareholderActualControllerHistory = BeanUtilsProxy.copyProperties(bocomShareholderActualController, BocomShareholderActualControllerHistory.class);
                bocomShareholderActualControllerHistory.setId(null);
                bocomShareholderActualControllerHistory.setBocomBasicInfoId(bocomBasicInfoHistory.getId());
                bocomShareholderActualControllerHistoryService.save(bocomShareholderActualControllerHistory);
                Long front = bocomShareholderActualController.getFront();
                if (front != null) {
                    FileStorageDO frontFile = fileService.getById(front);
                    frontFile.setId(null);
                    frontFile.setBusinessType(FileTypeEnum.BOCOM_SHAREHOLDER_IDCARD_FRONT_HISTORY.getCode());
                    frontFile.setBusinessId(bocomShareholderActualControllerHistory.getId());
                    fileService.save(frontFile);
                }
                Long back = bocomShareholderActualController.getBack();
                if (back != null) {
                    FileStorageDO backFile = fileService.getById(back);
                    backFile.setId(null);
                    backFile.setBusinessType(FileTypeEnum.BOCOM_SHAREHOLDER_IDCARD_BACK_HISTORY.getCode());
                    backFile.setBusinessId(bocomShareholderActualControllerHistory.getId());
                    fileService.save(backFile);
                }
            }
        }
        List<BocomBeneficiaryInfo> bocomBeneficiaryInfos = bocomBasicInfo.getBocomBeneficiaryInfos();
        if (CollectionUtil.isNotEmpty(bocomBeneficiaryInfos)) {
            for (BocomBeneficiaryInfo bocomBeneficiaryInfo : bocomBeneficiaryInfos) {
                BocomBeneficiaryInfoHistory bocomBeneficiaryInfoHistory = BeanUtilsProxy.copyProperties(bocomBeneficiaryInfo, BocomBeneficiaryInfoHistory.class);
                bocomBeneficiaryInfoHistory.setId(null);
                bocomBeneficiaryInfoHistory.setBocomBasicInfoId(bocomBasicInfoHistory.getId());
                bocomBeneficiaryInfoHistoryService.save(bocomBeneficiaryInfoHistory);
                Long front = bocomBeneficiaryInfo.getFront();
                if (front != null) {
                    FileStorageDO frontFile = fileService.getById(front);
                    frontFile.setId(null);
                    frontFile.setBusinessType(FileTypeEnum.BOCOM_BENEFICIARY_IDCARD_FRONT_HISTORY.getCode());
                    frontFile.setBusinessId(bocomBeneficiaryInfoHistory.getId());
                    fileService.save(frontFile);
                }
                Long back = bocomBeneficiaryInfo.getBack();
                if (back != null) {
                    FileStorageDO backFile = fileService.getById(back);
                    backFile.setId(null);
                    backFile.setBusinessType(FileTypeEnum.BOCOM_BENEFICIARY_IDCARD_BACK_HISTORY.getCode());
                    backFile.setBusinessId(bocomBeneficiaryInfoHistory.getId());
                    fileService.save(backFile);
                }
            }
        }
        BocomAuthorizedAgent bocomAuthorizedAgent = bocomBasicInfo.getBocomAuthorizedAgent();
        if (bocomAuthorizedAgent != null) {
            BocomAuthorizedAgentHistory bocomAuthorizedAgentHistory = BeanUtilsProxy.copyProperties(bocomAuthorizedAgent, BocomAuthorizedAgentHistory.class);
            bocomAuthorizedAgentHistory.setId(null);
            bocomAuthorizedAgentHistory.setBocomBasicInfoId(bocomBasicInfoHistory.getId());
            bocomAuthorizedAgentHistoryService.save(bocomAuthorizedAgentHistory);
            Long front = bocomAuthorizedAgent.getFront();
            if (front != null) {
                FileStorageDO frontFile = fileService.getById(front);
                frontFile.setId(null);
                frontFile.setBusinessType(FileTypeEnum.BOCOM_AUTHORIZED_AGENT_IDCARD_FRONT_HISTORY.getCode());
                frontFile.setBusinessId(bocomAuthorizedAgentHistory.getId());
                fileService.save(frontFile);
            }
            Long back = bocomAuthorizedAgent.getBack();
            if (back != null) {
                FileStorageDO backFile = fileService.getById(back);
                backFile.setId(null);
                backFile.setBusinessType(FileTypeEnum.BOCOM_AUTHORIZED_AGENT_IDCARD_BACK_HISTORY.getCode());
                backFile.setBusinessId(bocomAuthorizedAgentHistory.getId());
                fileService.save(backFile);
            }
        }
        return bocomBasicInfoHistory.getId();
    }
}
